#include <time.h>
#include "prodcons.h"
#include "linked_list.h"
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>
#include "mt64.h"

double start;
sem_t mutex;
sem_t full;
sem_t empty;
node * head_node;
const int NUM_WORKER_THREADS; //The number of worker threads.
const int NUM_SAMPLES_PER_THREAD; //This is the number of samples to geenerate
  //per thread.  It is possible to read in one value and store it in this 
  //constant.

const int NUM_ITERATIONS_PER_THREAD; //This is the number of iterations of 
  //sampling to perform per thread.

const double X_MIN_BOUND; //The lower bound for x.
const double X_MAX_BOUND; //The upper bound for x.
const double Y_MIN_BOUND; //The lower bound for y.
const double Y_MAX_BOUND; //The upper bound for y.

int main(int argc, char ** argv)
{
  unsigned long long init[4]={0x12345ULL, 0x23456ULL, 0x34567ULL, 0x45678ULL};
  unsigned long long length=4;
  init_by_array64(init, length);

  printf("Provide your input on one line in the following order:\n");
  printf("1:x minimum, 2:x maximum, 3:y minimum, 4:y maximum, 5:number of samples, \n");
  printf("6:number of iterations per thread, and 7:number of threads ");

  scanf("%lf %lf %lf %lf %d %d %d", &X_MIN_BOUND, &X_MAX_BOUND, &Y_MIN_BOUND,
    &Y_MAX_BOUND, &NUM_SAMPLES_PER_THREAD, &NUM_ITERATIONS_PER_THREAD, 
    &NUM_WORKER_THREADS);

  if(NUM_SAMPLES_PER_THREAD < 1)
  {
    printf("Number of samples per thread must be greater than zero\nExiting...\n");
    exit(0);
  }
  if(NUM_WORKER_THREADS < 1)
  {
    printf("Number of worker threads must be greater than zero\nExiting...\n");
    exit(0);
  }
  if(NUM_ITERATIONS_PER_THREAD < 1)
  {
    printf("Number of iterations per thread must be greater than zero\nExiting...\n");
    exit(0);
  }
  if(X_MIN_BOUND > X_MAX_BOUND)
  {
    printf("x maximum bound must be greater than x minimum bound\nExiting...\n");
    exit(0);
  }
  if(Y_MIN_BOUND > Y_MAX_BOUND)
  {
    printf("y maximum bound must be greater than y minimum bound\nExiting...\n");
    exit(0);
  }
  if(X_MIN_BOUND > -2000.0)
  {
    printf("x minimum bound must be less than or equal to -2000.0\nExiting...\n");
    exit(0);
  }
  if(Y_MIN_BOUND > -2000.0)
  {
    printf("y minimum bound must be less than or equal to -2.0\nExiting...\n");
    exit(0);
  }
  if(X_MAX_BOUND < 2000.0)
  {
    printf("x maximum bound must be greater than or equal to 2000.0\nExiting...\n");
    exit(0);
  }
  if(Y_MAX_BOUND < 2000.0)
  {
    printf("y maximum bound must be greater than or equal to 2000.0\nExiting...\n");
    exit(0);
  }

  pthread_t * producer_threads;
  pthread_t consumer_thread;
  head_node = NULL;

  sem_init(&mutex, 0, 1);
  sem_init(&full, 0, NUM_WORKER_THREADS*NUM_ITERATIONS_PER_THREAD);
  sem_init(&empty, 0, NUM_WORKER_THREADS*NUM_ITERATIONS_PER_THREAD);

  //Drain the full semaphore
  int i = 0;
  for(i = 0; i < NUM_WORKER_THREADS*NUM_ITERATIONS_PER_THREAD; i++)
    sem_wait(&full);

  printf("Created semaphores in main\n");

  int err_code;

  //Allocate thread variables.
  producer_threads = (pthread_t *) malloc(sizeof(pthread_t)*NUM_WORKER_THREADS);
  start = clock();

  for(i = 0; i < NUM_WORKER_THREADS; i++)
  {
    err_code = pthread_create(&producer_threads[i], NULL, producer, (void *) i);
    if(err_code != 0)
    {
      printf("Unable to create producer thread %d\n", i);
      exit(0);
    }
  }

  err_code = pthread_create(&consumer_thread, NULL, consumer, (void *) NUM_WORKER_THREADS);
  if(err_code != 0)
  {
    printf("Unable to create consumer thread\n");
    exit(0);
  }

  for(i = 0; i < NUM_WORKER_THREADS; i++)
  {
    err_code = pthread_join(producer_threads[i], NULL);
    if(err_code != 0)
    {
      printf("Unable to join producer thread %d\n", i);
      exit(0);
    }
  }

  err_code = pthread_join(consumer_thread, NULL);
  if(err_code != 0)
  {
    printf("Unable to join consumer thread\n");
    exit(0);
  }

  free((void *) producer_threads);
  sem_destroy(&mutex);
  sem_destroy(&full);
  sem_destroy(&empty);

  return 0;
}
